@@ -1,20 +1,27 @@
-## Copyright (C) 2005-2017 David Bateman
+########################################################################
+##
+## Copyright (C) 2005-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{d} =} eigs (@var{A})
@@ -72,32 +79,32 @@
 ## @var{sigma} is a string, it must have one of the following values.
 ##
 ## @table @asis
-## @item @qcode{"lm"}
+## @item @nospell{@qcode{"lm"}}
 ## Largest Magnitude (default).
 ##
-## @item @qcode{"sm"}
+## @item @nospell{@qcode{"sm"}}
 ## Smallest Magnitude.
 ##
 ## @item @qcode{"la"}
 ## Largest Algebraic (valid only for real symmetric problems).
 ##
-## @item @qcode{"sa"}
+## @item @nospell{@qcode{"sa"}}
 ## Smallest Algebraic (valid only for real symmetric problems).
 ##
 ## @item @qcode{"be"}
 ## Both Ends, with one more from the high-end if @var{k} is odd (valid only for
 ## real symmetric problems).
 ##
-## @item @qcode{"lr"}
+## @item @nospell{@qcode{"lr"}}
 ## Largest Real part (valid only for complex or unsymmetric problems).
 ##
-## @item @qcode{"sr"}
+## @item @nospell{@qcode{"sr"}}
 ## Smallest Real part (valid only for complex or unsymmetric problems).
 ##
-## @item @qcode{"li"}
+## @item @nospell{@qcode{"li"}}
 ## Largest Imaginary part (valid only for complex or unsymmetric problems).
 ##
-## @item @qcode{"si"}
+## @item @nospell{@qcode{"si"}}
 ## Smallest Imaginary part (valid only for complex or unsymmetric problems).
 ## @end table
 ##
@@ -124,8 +131,8 @@
 ## @item p
 ## The number of Lanzcos basis vectors to use.  More vectors will result in
 ## faster convergence, but a greater use of memory.  The optimal value of
-## @code{p} is problem dependent and should be in the range @var{k} to @var{n}.
-## The default value is @code{2 * @var{k}}.
+## @code{p} is problem dependent and should be in the range @code{@var{k} + 1}
+## to @var{n}.  The default value is @code{2 * @var{k}}.
 ##
 ## @item v0
 ## The starting vector for the algorithm.  An initial vector close to the
@@ -143,8 +150,8 @@
 ##
 ## @item permB
 ## The permutation vector of the Cholesky@tie{}factorization of @var{B} if
-## @code{cholB} is true.  That is @code{chol (@var{B}(permB, permB))}.  The
-## default is @code{1:@var{n}}.
+## @code{cholB} is true.  It is obtained by @code{[R, ~, permB] =
+## chol (@var{B}, @qcode{"vector"})}.  The default is @code{1:@var{n}}.
 ##
 ## @end table
 ##
@@ -204,40 +211,66 @@ function varargout = eigs (varargin)
 
   call_eig = false;
   offset = 0;
+  b = [];
   k = 6;
   sigma = "lm";
 
   if (isnumeric (varargin{1}) && issquare (varargin{1}))
     a = varargin{1};
-    if (nargin > 1 && isnumeric (varargin{2})
-        && issquare (varargin{2}) && size_equal (a, varargin{2}))
-      b = varargin{2};
-      offset = 1;
+    if (nargin > 1 && isnumeric (varargin{2}))
+      if (size_equal (a, varargin{2}))
+        b = varargin{2};
+        offset = 1;
+      elseif (isempty (varargin{2}))
+        ## Special syntax to do regular eigenvalue decomposition rather
+        ## than generalized eigenvalue decomposition (B = []).
+        offset = 1;
+      endif
     endif
 
-    if (rows (a) < 9)
+    if (rows (a) < 13)
       call_eig = true;
     endif
 
     if (nargin > 1 + offset)
-      tmp = varargin{2+offset};
-      if (isnumeric (tmp) && isscalar (tmp) && isreal (tmp)
-          && round (tmp) == tmp)
-        k = tmp;
-
-        if (rows (a) - k < 3)
-          call_eig = true;
-        endif
+      arg = varargin{2+offset};
+      if (isnumeric (arg) && isscalar (arg) && isreal (arg)
+          && fix (arg) == arg)
+        k = arg;
+        p = 2 * k;
+      elseif (isfield (arg, "p"))
+        p = arg.p;
+      endif
+      if (p >= rows (a))
+        call_eig = true;
       else
         call_eig = false;
       endif
 
       if (nargin > 2 + offset)
-        tmp = varargin{3+offset};
-        if (ischar (tmp))
-          sigma = tolower (tmp);
-        elseif (isnumeric (tmp) && isscalar (tmp))
-          sigma = tmp;
+        arg = varargin{3+offset};
+        if (ischar (arg))
+          sigma = tolower (arg);
+        elseif (isnumeric (arg) && isscalar (arg))
+          sigma = arg;
+        elseif (isfield (arg, "p"))
+          p = arg.p;
+        endif
+        if (p >= rows (a))
+          call_eig = true;
+        else
+          call_eig = false;
+        endif
+      endif
+      if (nargin > 3 + offset)
+        arg = varargin{4+offset};
+        if (isfield (arg, "p"))
+          p = arg.p;
+        else
+          p = 2 * k;
+        endif
+        if (p >= rows (a))
+          call_eig = true;
         else
           call_eig = false;
         endif
@@ -247,14 +280,14 @@ function varargout = eigs (varargin)
 
   if (call_eig)
     varargout = cell (1, min (2, max (1, nargout)));
-    if (offset)
-      real_valued = isreal (a) && isreal (b);
-      symmetric = issymmetric (a) && issymmetric (b);
-      [varargout{:}] = eig (a, b);
-    else
+    if (isempty (b))
       real_valued = isreal (a);
       symmetric = issymmetric (a);
       [varargout{:}] = eig (a);
+    else
+      real_valued = isreal (a) && isreal (b);
+      symmetric = issymmetric (a) && issymmetric (b);
+      [varargout{:}] = eig (a, b);
     endif
     varargout = select (varargout, k, sigma, real_valued, symmetric);
     if (nargout == 3)
@@ -287,53 +320,53 @@ function out = select (args, k, sigma, real_valued, symmetric)
         if (real_valued && symmetric)
           [~, idx] = sort (real (d), "descend");
         else
-          error ('eigs: sigma = "la" requires real symmetric problem');
+          error ('eigs: SIGMA = "la" requires real symmetric problem');
         endif
 
       case "sa"
         if (real_valued && symmetric)
           [~, idx] = sort (real (d), "ascend");
         else
-          error ('eigs: sigma = "sa" requires real symmetric problem');
+          error ('eigs: SIGMA = "sa" requires real symmetric problem');
         endif
 
       case "be"
         if (real_valued && symmetric)
           [~, idx] = sort (real (d), "ascend");
         else
-          error ('eigs: sigma = "be" requires real symmetric problem');
+          error ('eigs: SIGMA = "be" requires real symmetric problem');
         endif
 
       case "lr"
-        if (! (real_valued || symmetric))
+        if (! (real_valued && symmetric))
           [~, idx] = sort (real (d), "descend");
         else
-          error ('eigs: sigma = "lr" requires complex or unsymmetric problem');
+          error ('eigs: SIGMA = "lr" requires complex or unsymmetric problem');
         endif
 
       case "sr"
-        if (! (real_valued || symmetric))
+        if (! (real_valued && symmetric))
           [~, idx] = sort (real (d), "ascend");
         else
-          error ('eigs: sigma = "sr" requires complex or unsymmetric problem');
+          error ('eigs: SIGMA = "sr" requires complex or unsymmetric problem');
         endif
 
       case "li"
-        if (! (real_valued || symmetric))
+        if (! (real_valued && symmetric))
           [~, idx] = sort (imag (d), "descend");
         else
-          error ('eigs: sigma = "li" requires complex or unsymmetric problem');
+          error ('eigs: SIGMA = "li" requires complex or unsymmetric problem');
         endif
 
       case "si"
-        if (! (real_valued || symmetric))
+        if (! (real_valued && symmetric))
           [~, idx] = sort (imag (d), "ascend");
         else
-          error ('eigs: sigma = "si" requires complex or unsymmetric problem');
+          error ('eigs: SIGMA = "si" requires complex or unsymmetric problem');
         endif
 
       otherwise
-        error ("eigs: unrecognized value for sigma: %s", sigma);
+        error ("eigs: unrecognized value for SIGMA: %s", sigma);
     endswitch
   else
     ## numeric sigma, find k closest values
@@ -373,7 +406,7 @@ endfunction
 #### SPARSE MATRIX VERSIONS ####
 
 ## Real positive definite tests, n must be even
-%!shared n, k, A, d0, d2
+%!shared n, k, A, d0, d2, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[ones(1,n-2),4*ones(1,n),ones(1,n-2)]);
@@ -381,6 +414,8 @@ endfunction
 %! d2 = sort (d0);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -453,7 +488,7 @@ endfunction
 %! opts.issym = 1;  opts.isreal = 1;
 %! d1 = eigs (fn, n, k, 4.1, opts);
 %! assert (d1, eigs (A, k, 4.1), 1e-11);
-%!testif HAVE_ARPACK
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
 %! AA = speye (10);
 %! fn = @(x) AA * x;
 %! opts.issym = 1;  opts.isreal = 1;
@@ -490,13 +525,15 @@ endfunction
 %! endfor
 
 ## Real unsymmetric tests
-%!shared n, k, A, d0
+%!shared n, k, A, d0, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[ones(1,n-2),1:n,-ones(1,n-2)]);
 %! d0 = eig (A);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -617,13 +654,15 @@ endfunction
 
 
 ## Complex hermitian tests
-%!shared n, k, A, d0
+%!shared n, k, A, d0, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[1i*ones(1,n-2),4*ones(1,n),-1i*ones(1,n-2)]);
 %! d0 = eig (A);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -743,11 +782,129 @@ endfunction
 %! for i=1:k
 %!   assert (max (abs ((A - d1(i)*speye (n))*v1(:,i))), 0, 1e-11);
 %! endfor
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = toeplitz (sparse (1:10));
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, "lm");
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12)
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, "lm");
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10);
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, "lm");
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12);
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, "lm");
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10) -...
+%! 1i * spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10);
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, "lm");
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12);
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, "lm");
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = toeplitz (sparse (1:10));
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, 1);
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12);
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, 1);
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10);
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, 1);
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12);
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, 1);
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10) -...
+%! 1i * spdiags ([[1./(2:11)]',[-5:-2:-23]',[1:10]'],-1:1,10,10);
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [v, d] = eigs (A, B, 4, 1);
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12);
+%! endfor
+%! ddiag = diag (d);
+%! [ddiag, idx] = sort (ddiag);
+%! v = v(:, idx);
+%! R = chol (B);
+%! [v1, d1] = eigs (R' \ A / R, 4, 1);
+%! d1diag = diag (d1);
+%! [d1diag, idx] = sort (d1diag);
+%! v1 = v1(:, idx);
+%! assert (abs (v), abs (R \ v1), 1e-12);
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = toeplitz (sparse (1:10));
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! R = chol (B);
+%! opts.cholB = true;
+%! [v, d] = eigs (A, R, 4, "lm", opts);
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12)
+%! endfor
+%!testif HAVE_ARPACK, HAVE_UMFPACK, HAVE_CHOLMOD
+%! A = toeplitz (sparse (1:10));
+%! B = toeplitz (sparse ([1, 1], [1, 2], [2, 1], 1, 10));
+%! [R, ~, permB] = chol (B, "vector");
+%! opts.cholB = true;
+%! opts.permB = permB;
+%! [v, d] = eigs (A, R, 4, "lm", opts);
+%! for i = 1:4
+%!   assert (A * v(:,i), d(i, i) * B * v(:,i), 1e-12)
+%! endfor
+
 
 #### FULL MATRIX VERSIONS ####
 
 ## Real positive definite tests, n must be even
-%!shared n, k, A, d0, d2
+%!shared n, k, A, d0, d2, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = full (sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[ones(1,n-2),4*ones(1,n),ones(1,n-2)]));
@@ -755,6 +912,8 @@ endfunction
 %! d2 = sort (d0);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -859,13 +1018,15 @@ endfunction
 %! endfor
 
 ## Real unsymmetric tests
-%!shared n, k, A, d0
+%!shared n, k, A, d0, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = full (sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[ones(1,n-2),1:n,-ones(1,n-2)]));
 %! d0 = eig (A);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -985,13 +1146,15 @@ endfunction
 %! endfor
 
 ## Complex hermitian tests
-%!shared n, k, A, d0
+%!shared n, k, A, d0, old_state, restore_state
 %! n = 20;
 %! k = 4;
 %! A = full (sparse ([3:n,1:n,1:(n-2)],[1:(n-2),1:n,3:n],[1i*ones(1,n-2),4*ones(1,n),-1i*ones(1,n-2)]));
 %! d0 = eig (A);
 %! [~, idx] = sort (abs (d0));
 %! d0 = d0(idx);
+%! old_state = rand ("state");
+%! restore_state = onCleanup (@() rand ("state", old_state));
 %! rand ("state", 42); # initialize generator to make eigs behavior reproducible
 %!testif HAVE_ARPACK
 %! d1 = eigs (A, k);
@@ -1111,13 +1274,79 @@ endfunction
 %! for i=1:k
 %!   assert (max (abs ((A - d1(i)*eye (n))*v1(:,i))), 0, 1e-11);
 %! endfor
+%!testif HAVE_ARPACK
+%! [v1,d1] = eigs (A, k, "li");
+%! d1 = diag (d1);
+%! for i=1:k
+%!   assert (max (abs ((A - d1(i)*eye (n))*v1(:,i))), 0, 1e-11);
+%! endfor
 
-%!test
+%!testif HAVE_ARPACK
 %! A = 2 * diag (ones (10, 1)) - diag (ones (9, 1), 1) - diag (ones (9, 1), -1);
-%! B = diag (ones (10, 1));
+%! B = eye (10);
 %! reseig = eig (A, B);
 %! [~, idx] = sort (abs (reseig), "ascend");
-%! assert (eigs (A, B, 10, 0), reseig (idx));
+%! assert (eigs (A, B, 4, 0), reseig (idx(4:-1:1)), 8 * eps);
+%!testif HAVE_ARPACK
+%! A = eye (9);
+%! A(1, 1) = 0;
+%! A(1, 9) = 1;
+%! [V, L] = eigs (A, 4, -1);
+%! assert (!any (isnan (diag (L))));
+%! assert (any (abs (diag (L)) <= 2 * eps));
+%!testif HAVE_ARPACK
+%! A = diag (ones (9, 1), 1);
+%! A(10,:) = [-1, zeros(1, 8), -1];
+%! opts.v0 = (1:10)';
+%! typ = "lr";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (real (diag (m))), ...
+%!         [0.514038; 0.514038; 0.880290; 0.880290], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (real (m)), ...
+%!         [0.514038; 0.514038; 0.880290; 0.880290], 1e-4);
+%! typ = "li";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (abs (imag (diag (m)))), ...
+%!         [0.78972; 0.78972; 0.96518; 0.96518], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (abs (imag (m))), ...
+%!         [0.78972; 0.78972; 0.96518; 0.96518], 1e-4);
+%! typ = "sr";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (real (diag (m))), ...
+%!         [-1.12180; -1.12180; -0.69077; -0.69077], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (real (m)), ...
+%!         [-1.12180; -1.12180; -0.69077; -0.69077], 1e-4);
+%! typ = "si";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (abs (imag (diag (m)))), ...
+%!         [0.25552; 0.25552; 0.30282; 0.30282], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (abs (imag (m))), ...
+%!         [0.25552; 0.25552; 0.30282; 0.30282], 1e-4);
+%! typ = "lm";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (abs (diag (m))), ...
+%!         [1.02294;  1.02294; 1.15054; 1.15054], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (abs (m)), ...
+%!         [1.02294; 1.02294; 1.15054; 1.15054], 1e-4);
+%! typ = "sm";
+%! [v, m] = eigs (A, 4, typ, opts);
+%! assert (sort (abs (diag (m))), ...
+%!         [0.93092; 0.93092; 0.94228; 0.94228], 1e-4);
+%! m = eigs (A, 4, typ, opts);
+%! assert (sort (abs (m)), ...
+%!         [0.93092; 0.93092; 0.94228; 0.94228], 1e-4);
+%!testif HAVE_ARPACK
+%! A = toeplitz (sparse ([2, 1, zeros(1,8)]));
+%! opts.v0 = (1:10)';
+%! [v, m] = eigs (A, 3, "sa", opts);
+%! assert (diag (m), [0.081014; 0.317493; 0.690279], 1e-4);
+%! m = eigs (A, 3, "sa", opts);
+%! assert (m, [0.081014; 0.317493; 0.690279], 1e-4);
 
 %!test
 %! X = [70 47 42 39 50 73 79 23;
@@ -1133,11 +1362,231 @@ endfunction
 %! Z = X * X';
 %! r = rank (Z);
 %! assert (r, 8);
-%! [V, D] = eigs (Z, r, "lm");
+%! [V, D] = eigs (Z, r, "lm"); # call_eig is true
 %! ZZ = V * D * V';
 %! tmp = abs (Z - ZZ);
 %! assert (max (tmp(:)) < 5e-11);
 
-%!assert (eigs (diag (1:5), 5, "sa"), [1;2;3;4;5])
-%!assert (eigs (diag (1:5), 5, "la"), [5;4;3;2;1])
-%!assert (eigs (diag (1:5), 3, "be"), [1;4;5])
+%!assert (eigs (diag (1:5), 5, "sa"), [1;2;3;4;5]) # call_eig is true
+%!assert (eigs (diag (1:5), 5, "la"), [5;4;3;2;1]) # call_eig is true
+%!assert (eigs (diag (1:5), 3, "be"), [1;4;5]) # call_eig is true
+%!error
+%! A = rand (10);
+%! opts.v0 = ones (8, 1);
+%! eigs (A, 4, "sm", opts);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([-2, 1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 3;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 4, "lm", opts);
+%! assert (d(3:4), [NaN; NaN]);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([-2, 1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 1;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 4, "sm", opts);
+%! assert (d(4), NaN);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([-2, 1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! Afun = @(x) A * x;
+%! opts.v0 = (1:100)';
+%! opts.maxit = 3;
+%! opts.issym = true;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (Afun, 100, 4, "sm", opts);
+%! assert (d(3:4), [NaN; NaN]);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([-2, 1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! A(1, 2) = 10;
+%! opts.v0 = (1:100)';
+%! opts.maxit = 5;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 4, "lm", opts);
+%! assert (d(3:4), [NaN; NaN]);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([0, 1, zeros(1, 8)], [0, -1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 4;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 4, "lm", opts);
+%! assert (d(3:4), [NaN+1i*NaN; NaN+1i*NaN]);
+%!testif HAVE_ARPACK
+%! A = magic (100);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 1;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 10, "lm", opts);
+%! assert (d(9:10), [NaN; NaN]);
+%!testif HAVE_ARPACK
+%! A = toeplitz ([0, 1, zeros(1, 8)], [0, -1, zeros(1, 8)]);
+%! A(1, 1) = 1;
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 1;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 4, "sm", opts);
+%! if (isreal (d))
+%!   assert (d(4), NaN);
+%! else
+%!   assert (d(4), NaN +1i*NaN);
+%! endif
+%!testif HAVE_ARPACK
+%! A = magic (100) / 10 + eye (100);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 10;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 10, "sm", opts);
+%! if (isreal (d))
+%!   assert (d(10), NaN);
+%! else
+%!   assert (d(10), NaN +1i*NaN);
+%! endif
+%!testif HAVE_ARPACK
+%! A = toeplitz ([0, 1, zeros(1, 8)], [0, -1, zeros(1, 8)]);
+%! A = kron (A, eye (10)) + kron (eye (10), A);
+%! Afun = @(x) A * x;
+%! opts.v0 = (1:100)';
+%! opts.maxit = 4;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (Afun, 100, 4, "lm", opts);
+%! assert (d(3:4), [NaN+1i*NaN; NaN+1i*NaN]);
+%!testif HAVE_ARPACK
+%! A = 1i * magic (100);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 1;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 5, "lm", opts);
+%! assert (d(5), NaN+1i*NaN);
+%!testif HAVE_ARPACK
+%! A = 1i * magic (100) + eye (100);
+%! opts.v0 = (1:100)';
+%! opts.maxit = 7;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (A, 10, "sm", opts);
+%! assert (d(9:10), [NaN+1i*NaN; NaN+1i*NaN]);
+%!testif HAVE_ARPACK
+%! A = 1i * magic (100);
+%! Afun = @(x) A * x;
+%! opts.v0 = (1:100)';
+%! opts.maxit = 1;
+%! opts.isreal = false;
+%! warning ("off", "Octave:eigs:UnconvergedEigenvalues", "local");
+%! d = eigs (Afun, 100, 6, "lm", opts);
+%! assert (d(6), NaN+1i*NaN);
+%!testif HAVE_ARPACK, HAVE_CHOLMOD
+%! A = sparse (magic (10));
+%! B = sparse (magic (10)); # not HPD
+%! fail ("eigs (A, B, 4)", "eigs: The matrix B is not positive definite")
+%!testif HAVE_ARPACK
+%! i_A = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
+%! j_A = [1, 2, 3, 4, 5, 6, 7,  8, 9, 10];
+%! v_A = [1, 2i, 3, 4i, 5, 6i, 7, 8, 9, 10i];
+%! A = sparse(i_A, j_A, v_A);
+%! i_B = [1,2, 3, 4, 5, 6, 7, 8, 9, 10];
+%! j_B = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
+%! v_B = [3, 10i, 1, 8i, 7, 6i, 5, 4i, 9, 7i];
+%! B = sparse(i_B, j_B, v_B); # not SPD
+%! [Evectors, Evalues] = eigs(A, B, 5, "SM"); # call_eig is true
+%! ResidualVectors = A * Evectors - B * Evectors * Evalues;
+%! RelativeErrors = norm (ResidualVectors, "columns") ./ ...
+%! norm (A * Evectors, "columns");
+%! assert (RelativeErrors, zeros (1, 5));
+%!testif HAVE_ARPACK
+%! A = rand (8);
+%! eigs (A, 6, "lr"); # this failed on 4.2.x
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M);
+%! B = cos (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "LM", opts);
+%! Afun = @(x) A * x;
+%! [Evector_f Evalues_f] = eigs (Afun, 10, B, 4, "LM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M);
+%! B = cos (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "SM", opts);
+%! [L, U, P] = lu (A);
+%! Afun = @(x) U \ (L \ (P * x));
+%! [Evector_f Evalues_f] = eigs (Afun, 10, B, 4, "SM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M);
+%! A = A * A';
+%! B = cos (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "LM", opts);
+%! Afun = @(x) A * x;
+%! opts.issym = true;
+%! [Evector_f Evalues_f] = eigs (Afun, 10, B, 4, "LM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M);
+%! A = A * A';
+%! B = cos (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "SM", opts);
+%! [L, U, P] = lu (A);
+%! Afun = @(x) U \ (L \ (P * x));
+%! opts.issym = true;
+%! [Evector_f Evalues_f] = eigs (Afun, 10, B, 4, "SM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M) + 1i * cos (M);
+%! B = cos (M) + 1i * sin (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "LM", opts);
+%! Afun = @(x) A * x;
+%! opts.isreal = false;
+%! [Evector_f Evalues_f] = eigs (Afun, 10, B, 4, "LM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+%!testif HAVE_ARPACK
+%! M = magic (10);
+%! A = sin (M) + 1i * cos (M);
+%! B = cos (M) + 1i * sin (M);
+%! B = B * B';
+%! opts.v0 = (1:10)';
+%! [Evector, Evalues] = eigs (A, B, 4, "SM", opts);
+%! [L, U, P] = lu (A);
+%! Afun = @(x) U \ (L \ (P *x));
+%! opts.isreal = false;
+%! [Evector_f, Evalues_f] = eigs (Afun, 10, B, 4, "SM", opts);
+%! assert (Evector, Evector_f);
+%! assert (Evalues, Evalues_f);
+
+%!testif HAVE_ARPACK <*57196>
+%! x = ones (10, 10);
+%! z = complex (x, x);
+%! A = [sparse(10,10), z; z', sparse(10,10)];
+%! d = eigs (A);
+%! assert (isreal (d));
+%! [~, d] = eigs (A);
+%! assert (isreal (d));
+
+%!testif HAVE_ARPACK <*59486>
+%! A = magic (5);
+%! d = eigs (A, [], 1);
+%! assert (d, 65, 5 * eps (65));
